@@ -1,6 +1,6 @@
 package Net::OSCAR::Connection;
 
-$VERSION = 0.59;
+$VERSION = 0.60;
 
 use strict;
 use vars qw($VERSION);
@@ -53,9 +53,11 @@ sub flap_encode($$;$) {
 sub flap_put($;$$) {
 	my($self, $msg, $channel) = @_;
 	my $emsg;
+	my $had_outbuff = 0;
 
 	return unless $self->{socket} and CORE::fileno($self->{socket}) and getpeername($self->{socket}); # and !$self->{socket}->error;
 
+	$had_outbuff = 1 if $self->{outbuff};
 	if($msg) {
 		$emsg = $self->flap_encode($msg, $channel);
 		$self->{outbuff} .= $emsg;
@@ -69,7 +71,12 @@ sub flap_put($;$$) {
 		return undef;
 	} else {
 		$emsg = substr($self->{outbuff}, 0, $nchars, "");
-		$self->log_print(OSCAR_DBG_NOTICE, "Couldn't do complete write - had to buffer ", length($self->{outbuff}), " bytes.") if $self->{outbuff};
+		if($self->{outbuff}) {
+			$self->log_print(OSCAR_DBG_NOTICE, "Couldn't do complete write - had to buffer ", length($self->{outbuff}), " bytes.");
+			$self->{session}->callback_connection_changed($self, "readwrite");
+		} elsif($had_outbuff) {
+			$self->{session}->callback_connection_changed($self, "read");
+		}
 		$self->log_print(OSCAR_DBG_PACKETS, "Put ", hexdump($emsg));
 	}
 }
@@ -243,13 +250,22 @@ sub connect($$) {
 sub get_filehandle($) { shift->{socket}; }
 
 # $read/$write tell us if select indicated readiness to read and/or write
-sub process_one($$$) {
-	my($self, $read, $write) = @_;
+# Dittor for $error
+sub process_one($;$$$) {
+	my($self, $read, $write, $error) = @_;
 	my $snac;
 	my %tlv;
 
+	if($error) {
+		$self->{sockerr} = 1;
+		return $self->disconnect();
+	}
+
 	tie %tlv, "Net::OSCAR::TLV";
 
+	$read ||= 1;
+	$write ||= 1;
+
 	if($write && $self->{outbuff}) {
 		$self->log_print(OSCAR_DBG_DEBUG, "Flushing output buffer.");
 		$self->flap_put();
@@ -258,7 +274,6 @@ sub process_one($$$) {
 	if($write && !$self->{connected}) {
 		$self->log_print(OSCAR_DBG_NOTICE, "Connected.");
 		$self->{connected} = 1;
-		#$self->set_blocking(1);
 		$self->{session}->callback_connection_changed($self, "read");
 		return 1;
 	} elsif($read && !$self->{ready}) {
@@ -343,4 +358,7 @@ sub ready($) {
 		));
 	}
 }
+
+sub session($) { return shift->{session}; }
+
 1;
